@@ -1,7 +1,6 @@
 package SQL::OrderBy;
 
 use strict;
-use vars '$VERSION'; $VERSION = '0.08.7';
 use base 'Exporter';
 use vars qw(@EXPORT @EXPORT_OK);
 @EXPORT_OK = @EXPORT = qw(
@@ -10,34 +9,38 @@ use vars qw(@EXPORT @EXPORT_OK);
     col_dir_list
     to_asc_desc
 );
+use vars qw($VERSION);
+$VERSION = '0.08';
 
+# sub toggle_resort {{{
 # Transform an order by clause.
-sub toggle_resort {  # {{{
+sub toggle_resort {
     my %args = @_;
 
     # Set the column name list and the directions.
-    my ($columns, $direction, $asc_desc) = get_columns(
+    my ($columns, $direction) = get_columns(
         %args,
         name_direction => 1,
         numeric_direction => 1,
     );
 
     # Handle a selected column.
-    if (my $name = $args{selected}) {
-        ($name, $direction, $asc_desc) = _name_direction(
-            $name, $direction, $asc_desc
-        );
+    if (my $selected = $args{selected}) {
+        # Set the direction according to our show_ascending flag.
+        $direction->{$selected} =
+            !$direction->{$selected} || $direction->{$selected}
+                ? 1 : 0;
 
         # Toggle if the selected column is the first one.
-        if ($name eq $columns->[0]) {
-            $direction->{$name} = $direction->{$name}
+        if ($selected eq $columns->[0]) {
+            $direction->{$selected} = $direction->{$selected}
                 ? 0 : 1;
         }
 
         # Remove the selected column name from its old position.
-        @$columns = grep { $_ ne $name } @$columns;
+        @$columns = grep { $_ ne $selected } @$columns;
         # And add the selected column name to the beginning.
-        unshift @$columns, $name;
+        unshift @$columns, $selected;
     }
 
     # Convert from numeric, if asked to.
@@ -49,43 +52,43 @@ sub toggle_resort {  # {{{
 
     # Return the column ordering as an arrayref or string.
     return wantarray ? @$columns : join ', ', @$columns;
-}  # }}}
+}
+# }}}
 
+# sub get_columns {{{
 # Return the column names and directions as either hash/array
 # references, or a column array, or an "order by" clause.
-sub get_columns {  # {{{
+sub get_columns {
     my %args = @_;
 
-    # Set the order array from the order_by argument.
-    my @order;
-    if (ref $args{order_by} eq 'ARRAY') {
-        @order = @{ $args{order_by} };
-#        warn "Empty order list provided." unless @order;
-    }
-    else {
-        if ($args{order_by}) {
-            # Strip off any unneeded SQL clauses.
-            $args{order_by} =~ s/^.*?\border by\s+(.*)$/$1/i;
-            # Split the order clause.
-            @order = split /\s*,\s*/, $args{order_by};
-        }
-        else {
-            @order = ();
-#            warn "No statement or clause provided.\n" unless $args{order_by};
-        }
-    }
+    # Bail out unless we are given an order by clause or full SQL.
+    die "No statement or clause provided.\n" unless $args{order_by};
+
+    # Strip off any unneeded SQL clauses.
+    $args{order_by} =~ s/^.*?\border by\s+(.*)$/$1/i;
 
     # Hold column names and directions.
-    my ($columns, $direction, $asc_desc);
+    my ($columns, $direction);
+
+    # The ordering was sent in as either a list or a clause.
+    my @order = ref $args{order_by} eq 'ARRAY'
+        ? @{ $args{order_by} }
+        : split /\s*,\s*/, $args{order_by};
 
     # Set the column array and direction hashes.
+    my $asc_desc = {};
     for (@order) {
-        (my $name, $direction, $asc_desc) = _name_direction(
-            $_, $direction, $asc_desc
-        );
+        if (/^(.*?)(?:\s+(asc|desc))?$/i) {
+            # Use the direction provided; Ascend by default.
+            my ($name, $dir) = ($1, $2);
+            # Set the numeric directions.
+            $direction->{$name} = $dir && lc ($dir) eq 'desc' ? 0 : 1;
+            # Set the case sensitive alpha directions.
+            $asc_desc->{$name} = $dir ? $dir : '';
+        }
 
         # Add the column to our columns array.
-        push @$columns, $name;
+        push @$columns, $1;
     }
 
     # Make alpha directions if asked to.
@@ -94,29 +97,31 @@ sub get_columns {  # {{{
 
     # NOTE: name_direction only makes sense in an array context.
     if ($args{name_direction}) {
-        $columns = [ $columns, $direction, $asc_desc ];
+        $columns = [ $columns, $direction ];
     }
     else {
         @$columns = col_dir_list ($columns, $direction);
     }
 
     return wantarray ? @$columns : join ', ', @$columns;
-}  # }}}
+}
+# }}}
 
-# Return an array of column names with their respective directions
-# concatinated.  This is conditional concatination.  ASC/DESC vs.
-# 1/0 issues do not concern us here.
-sub col_dir_list {  # {{{
+# sub col_dir_list {{{
+# Return an array of column names with their respective directions concatinated.
+sub col_dir_list {
     my ($columns, $direction) = @_;
     return map {
         $direction->{$_}
             ? "$_ $direction->{$_}"
             : $_
     } @$columns;
-}  # }}}
+}
+# }}}
 
+# sub to_asc_desc {{{
 # Return alpha directions in place of numeric eqivalents.
-sub to_asc_desc {  # {{{
+sub to_asc_desc {
     my $dir = shift;
     my %args = @_;
 
@@ -143,57 +148,20 @@ sub to_asc_desc {  # {{{
     }
 
     return %$dir;
-}  # }}}
-
-sub _name_direction {  # {{{
-    my ($col, $direction, $asc_desc) = @_;
-
-    if ($col =~ /^(.*?)(?:\s+(asc|desc))?$/i) {
-        # Use the direction provided; Ascend by default.
-        ($col, my $dir) = ($1, $2);
-        # Set the numeric directions.
-        $direction->{$col} = $dir && lc ($dir) eq 'desc' ? 0 : 1;
-        # Set the case sensitive alpha directions.
-        $asc_desc->{$col} = $dir ? $dir : '';
-    }
-
-    return $col, $direction, $asc_desc;
-}  # }}}
+}
+# }}}
 
 1;
 __END__
 
 =head1 NAME
 
-SQL::OrderBy - Transform an SQL "order by" clause.
+SQL::OrderBy - Transform an SQL ORDER BY clause.
 
 =head1 SYNOPSIS
 
   use SQL::OrderBy;
 
-  @order = toggle_resort (
-      show_ascending => 1,
-      selected => 'artist',
-      order_by => ['name', 'artist', 'album'],
-  );
-  # ('artist asc', 'name asc', 'album asc')
-
-  print scalar toggle_resort (
-      selected => 'time',
-      order_by => scalar toggle_resort(
-          selected => 'artist',
-          order_by => scalar toggle_resort(
-              selected => 'artist',
-              order_by => 'name asc, artist asc, album asc'
-          )
-      )
-  );
-  # 'time, artist desc, name, album'
-
-  #----------------------------------------------------------------
-  # The following are functions used by the toggle_resort function,
-  # but are probably handy, in their own right.  : )
-
   # Fetch the columns in array context.
   @columns = get_columns (
       order_by => 'Name, Artist Desc, Album',
@@ -208,30 +176,51 @@ SQL::OrderBy - Transform an SQL "order by" clause.
   );
   # 'NAME ASC, ARTIST DESC, ALBUM ASC'
 
-  # Fetch the columns as a name array and direction hashes.
+  # Fetch the columns as a name array and numeric direction hash.
   @columns = get_columns (
-      order_by => 'name, artist deSc, album',
+      order_by => 'name, artist desc, album',
       name_direction    => 1,
       numeric_direction => 1,
   );
-  # (['name','artist','album'],
-  #  {name=>1, artist=>0, album=>1},
-  #  {name=>'', artist=>'deSc', album=>''})
+  # (['name','artist','album'], {name=>1, artist=>0, album=>1})
 
   # Output a "column direction" array.
   @columns = col_dir_list (\@column_names, \%direction);
   # ('name', 'artist desc', 'album')
 
-  # Convert the numeric directions to SQL keywords.
-  %direction = to_asc_desc (
-      \%direction,
-      show_ascending => 1,
-  );
+  # Fetch the directions as SQL keywords.
+  %direction = to_asc_desc (\%direction);
   # (name=>'asc', artist=>'desc', album=>'asc')
 
+  # Single toggle resort in array context.
+  @order = toggle_resort (
+      show_ascending => 1,
+      selected => 'artist',
+      order_by => ['name', 'artist', 'album'],
+  );
+  # ('artist asc', 'name asc', 'album asc')
+
+  # Nested toggle resort in scalar context.
+  print scalar toggle_resort (
+      selected => 'time',
+      order_by => scalar toggle_resort(
+          selected => 'artist',
+          order_by => scalar toggle_resort(
+              selected => 'artist',
+              order_by => 'name asc, artist asc, album asc'
+          )
+      )
+  );
+  # 'time, artist desc, name, album'
+
+=head1 ABSTRACT
+
+Resort and toggle (ascending/descending) table columns given an SQL
+ORDER BY clause.
+
 =head1 DESCRIPTION
 
-This package simply transforms an SQL "order by" clause by moving or
+This package simply transforms an SQL ORDER BY clause by moving or
 adding column names and toggling their ascending/descending state.
 
 Note that this is intentionally naive code, in that no database
@@ -241,57 +230,32 @@ integrity checking is done.
 
 =head2 toggle_resort
 
-  @columns = toggle_resort(
+  toggle_resort(
       order_by => $order_clause_or_list,
       selected => $column_name,
-      # Optional arguments:
-      show_ascending    => $w,
-      uc_direction      => $x,
-      name_direction    => $y,
-      numeric_direction => $z,
-  );
-
-  $columns = toggle_resort(
-      order_by => $order_clause_or_list,
-      selected => $column_name,
-      # Optional arguments:
-      show_ascending    => $w,
-      uc_direction      => $x,
   )
 
 This function takes two arguments provided as named parameters: an 
-SQL "order by" clause (provided as either a string or an array 
-reference) and a column name.
+SQL "ORDER BY" clause as either a string or array reference and a 
+column name.
 
 The selected column name is moved or added to the beginning of the
 clause with its sort direction exposed.  If this column is the first
 column of the list, its sort direction is flipped between ascending
 (asc) and descending (desc).
 
-In a scalar context, this function returns the clause as a (CSV)
-string.  In an array context, this function returns a list of column
-names with their respective sort directions.
-
 Note that the state of the sort is maintained, since the selected
 column name is the only one that is fondled.
 
-This function implements an essential feature for GUI environments, 
-where the user interacts with a database table by sorting and 
-resorting with (for instance) a mouse and "toggle button column 
-headings", during an interactive search refinement session.
-
-* If you do not include the selected argument, this function will
-simply return the clause with sort directions for each column name.
-That is, no "toggling" or moving is done.
+In a scalar context, this function returns the clause as a (CSV)
+string.  In an array context, this function returns a list of column
+names with their respective sort directions.
 
 This function optionally takes Boolean flags affecting the returned
 data structure.  These are:
 
 show_ascending => Expose the asc column directions.  Off by default.
 
-uc_direction => Render any new alpha column direction in uppercase.
-Off by default.
-
 name_direction => Return references to the column names and their
 directions.  Off by default.  Only makes sense in array context.
 
@@ -299,23 +263,33 @@ numeric_direction => Return Boolean column directions, instead of
 asc/desc.  Off by default.  Only makes sense with the name_direction
 flag on.
 
+uc_direction => Render any new alpha column direction in uppercase.
+Off by default.
+
+This implements an essential feature for GUI environments, where the
+user interacts with a table by sorting and resorting with a mouse and
+"toggle button column headings" during an interactive search
+refinement session.
+
+* If you do not include the selected argument, this function will
+simply return the clause with sort directions for each column name.
+That is, no "toggling" or moving is done.
+
 =head2 get_columns
 
   @columns = get_columns (
       order_by => $order_clause_or_list,
-      # Optional arguments:
-      show_ascending    => $w,
-      uc_direction      => $x,
+      uc_direction      => $w,
+      show_ascending    => $x,
       name_direction    => $y,
       numeric_direction => $z,
-  );
+  )
 
   $columns = get_columns (
       order_by => $order_clause_or_list,
-      # Optional arguments:
-      show_ascending => $w,
-      uc_direction   => $x,
-  );
+      show_ascending => $x,
+      uc_direction   => $y,
+  )
 
 This function simply returns a well formed order by clause or list.
 It can accept either a string or array reference for the order_by
@@ -323,11 +297,22 @@ argument.
 
 In a scalar context, this function returns the clause as a (CSV)
 string.  In an array context, this function returns a list of column
-names with their respective sort directions as numeric hash and a
-hash of the exact sort directions passed in.
+names with their respective sort directions.
 
-* The optional arguments are described in the toggle_resort function
-documentation, above.
+This function optionally takes Boolean flags affecting the returned
+data structure.  These are:
+
+show_ascending => Expose the asc column directions.  Off by default.
+
+name_direction => Return references to the column names and their
+directions.  Off by default.  Only makes sense in array context.
+
+numeric_direction => Return Boolean column directions, instead of
+asc/desc.  Off by default.  Only makes sense with the
+name_direction flag on.
+
+uc_direction => Render any new alpha column direction in uppercase.
+Off by default.
 
 =head2 col_dir_list
 
@@ -343,9 +328,8 @@ reference to a direction hash.
 
   %direction = to_asc_desc (
       \%direction,
-      # Optional arguments:
-      show_ascending => $w,
-      uc_direction   => $x,
+      show_ascending => $x,
+      uc_direction   => $y,
   )
 
 Return column directions as alpha keywords in place of their numeric
@@ -354,14 +338,20 @@ eqivalents.
 If the direction hash contains any alpha (asc/desc) values, the
 function uses those by default.
 
-* The optional arguments are described in the toggle_resort function
-documentation, above.
+This function takes a reference to a direction hash and optional
+flags to control the display of the asc keyword and new direction
+names.  These flags are:
+
+show_ascending => Expose the asc column directions.  Off by default.
+
+uc_direction => Render any new alpha column direction in uppercase.
+Off by default.
 
 =head1 DEPENDENCIES
 
 None.
 
-=head1 TO DO
+=head1 TODO
 
 Add functions for different kinds of resorting?
 
@@ -371,7 +361,7 @@ See the Changes file in this distribution.
 
 =head1 AUTHOR
 
-Gene Boggs E<lt>cpan@ology.netE<gt>
+Gene Boggs, E<lt>cpan@ology.netE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
